
#include <stdlib.h>			/* for free */

#include <evblib/evproc/evproc.h>


/* This creates empty evproc functions prefixed with null_. Convenient if
   you don't need to define a particular function yourself. */
 
#define F(type, name, args...)	type null_##name( args ) { return (type)0; }
#include <evblib/evproc/evproc.inc>
#undef F


void evprocset_init(EVPROCSET_T *eps)
{
    FD_ZERO(&eps->rfds);
    FD_ZERO(&eps->wfds);
    eps->firsttimer = 0;
    eps->highestfd = -1;
    eps->ephead = eps->eptail = 0;
}


void evprocset_add(EVPROCSET_T *eps, EVPROC_T *ep)
{
    LIST_ADDTAIL(&eps->ephead, &eps->eptail, ep);
}


void evprocset_rem(EVPROCSET_T *eps, EVPROC_T *ep)
{
    LIST_DEL(&eps->ephead, &eps->eptail, ep);
}


int evprocset_update(EVPROCSET_T *eps)
{
    EVPROC_T *ep;
    time_t t;
    int ret, hfd, n;

    ret = 0;
    eps->highestfd = -1;
    eps->firsttimer = 0;
    for(ep = eps->ephead; ep; ep = ep->next)
    {
        n = ep->ep_getfds(ep, &hfd, &eps->rfds, &eps->wfds, &t);
	if (!n) continue;
	ret++;
	if (hfd > eps->highestfd) 
		eps->highestfd  = hfd;
	if (t > 0 && (eps->firsttimer == 0 || t < eps->firsttimer)) 
		eps->firsttimer = t;
    }
    return ret;
}


int evprocset_handle(EVPROCSET_T *eps, int fdcnt, fd_set *rfds, fd_set *wfds, 
		     time_t t)
{
    EVPROC_T *ep;
    int ret;
    
    for(ep = eps->ephead, ret = 0; ep; ep = ep->next)
	ret |= ep->ep_handle(ep, fdcnt, rfds, wfds, t);
    return ret;
}


void evprocset_done(EVPROCSET_T *eps)
{
    EVPROC_T *ep, *nextep;

    for(ep = eps->ephead; ep; ep = nextep) {
	nextep = ep->next;
	ep->ep_done(ep);
	free(ep);
    }
    evprocset_init(eps);
}

