#ifndef _EVPROC_H
#define _EVPROC_H 1

/* INCLUDES & DEFINES */

#include <sys/types.h>
#include <sys/select.h>
#include <time.h>

#include <evblib/db/list.h>


/* TYPES */

/* evproc defines is an interface between event driven subprocesses and a 
   main loop that multiplexes those events. */

typedef struct evproc {

	/* The function pointers */

#define F(type, name, args...)	type (*name)( args );
#include <evblib/evproc/evproc.inc>
#undef F
	
	/* Local stuff */

	void *private;

	LISTABLE(struct evproc);

} EVPROC_T;


typedef struct evprocset {

    fd_set rfds, wfds;
    int highestfd;
    time_t firsttimer;
    EVPROC_T *ephead, *eptail;

} EVPROCSET_T;


/* PROTOTYPES */

/* This creates prototypes for the empty evproc functions, which are prefixed
   with null_. Convenient if you don't need to define a particular function 
   yourself. */
 
#define F(type, name, args...)	type null_##name( args );
#include <evblib/evproc/evproc.inc>
#undef F


void evprocset_init(EVPROCSET_T *eps);

void evprocset_add(EVPROCSET_T *eps, EVPROC_T *ep);   /* transfers ownership */
void evprocset_rem(EVPROCSET_T *eps, EVPROC_T *ep);   /* transfers it back */

/* returns nonzero if the set contains processes that were interested in 
   anything */

int evprocset_update(EVPROCSET_T *eps);

/* returns nonzero if an update should be done before waiting again */

int evprocset_handle(EVPROCSET_T *eps, int fdcnt, fd_set *rfds, fd_set *wfds, 
	      	     time_t t);

/* calls evproc_done and frees all evprocs in the set */

void evprocset_done(EVPROCSET_T *eps);

#endif
