
/* This is a header fragment that defines the event process interface. 
   Can be used to define a set of function pointers or customized prototypes, 
   as follows:
   
     #define F(type, name, args...)	type (*name)( args );
     #include <evblib/evproc/evproc.inc
     #undef F

   defines function pointers, which you also get nicely packaged in
   EVPROC_T, and

     #define F(type, name, args...)	type prefix_##name( args );
     #include <evblib/evproc/evproc.inc
     #undef F

   defines a set of prototypes where each function is prefixed by 'prefix_'.
*/


/* init initialises an evproc structure. You can initialize the function 
   pointers first and then have your own init do the rest later. */

F(void,		ep_init,	struct evproc *ep)

/* getfds adds the filedescriptors this process is interested in to the
   rfds and wfds sets and returns its highest file descriptor in highestfd
   and its first expiring timer in firsttimer. If it's not interested in
   anything, it returns 0 and leaves its return variables untouched, otherwise 
   it returns 1 and unconditionally overwrites *highestfd and *firsttimer,
   regardless of previous content. */
   
F(int,		ep_getfds,	struct evproc *ep, int *highestfd, 
				fd_set *rfds, fd_set *wfds, time_t *firsttimer)

/* handle tests the file descriptor sets, count and the current time time to
   see if anything is to be done. Must be prepared for spurious wakeups and
   allow for zero rfds and wfds pointers, as well as a zero value for 't'.
   A zero rfds/wfds must be interpreted as a given set with all descriptors
   set to 1; it should cause the subprocess to attempt an input/output cycle.
   It returns a bitmap where bit 0 indicates whether the event has caused a
   change in the process' status so that a new getfds should be done on 
   itself, and bit 1 indicates whether it has caused a change in any other
   subprocess' status, so that an update of the whole procset should be done */

F(int,		ep_handle,	struct evproc *ep, int fdcnt, 
				fd_set *rfds, fd_set *wfds, time_t t)

/* can be used to clean up whatever data is attached to private or by directly
   deriving from struct evproc. */

F(void,		ep_done,	struct evproc *ep)

